@@ -6,7 +6,19 @@
 {-# LANGUAGE DeriveFunctor #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE ScopedTypeVariables #-}
--- | Please see the README.md file for examples of using this API.
+-- | The simplest way to get started with this API is to turn on
+-- @OverloadedStrings@ and call 'runProcess'.  The following will
+-- write the contents of @/home@ to @stdout@ and then print the exit
+-- code (on a UNIX system).
+--
+-- @
+-- {-\# LANGUAGE OverloadedStrings \#-}
+--
+-- 'runProcess' "ls -l /home" >>= print
+-- @
+--
+-- Please see the [README.md](https://github.com/fpco/typed-process#readme)
+-- file for more examples of using this API.
 module System.Process.Typed
     ( -- * Types
       ProcessConfig
@@ -19,6 +31,8 @@ module System.Process.Typed
     , proc
     , shell
 
+      -- | #processconfigsetters#
+
       -- ** Setters
     , setStdin
     , setStdout
@@ -42,8 +56,10 @@ module System.Process.Typed
     , setChildUserInherit
 #endif
 
+      -- | #streamspecs#
+
       -- * Stream specs
-    , mkStreamSpec
+      -- ** Built-in stream specs
     , inherit
     , nullStream
     , closed
@@ -53,25 +69,35 @@ module System.Process.Typed
     , useHandleOpen
     , useHandleClose
 
+    -- ** Create your own stream spec
+    , mkStreamSpec
+    , mkPipeStreamSpec
+
+      -- | #launchaprocess#
+
       -- * Launch a process
+    , runProcess
+    , readProcess
+    , readProcessStdout
+    , readProcessStderr
+    , readProcessInterleaved
+    , withProcessWait
+    , withProcessTerm
     , startProcess
     , stopProcess
-    , withProcessWait
-    , withProcessWait_
-    , withProcessTerm
-    , withProcessTerm_
-    , withProcess
-    , withProcess_
-    , readProcess
-    , readProcess_
-    , runProcess
+      -- ** Exception-throwing functions
+      -- | The functions ending in underbar (@_@) are the same as
+      -- their counterparts without underbar but instead of returning
+      -- an 'ExitCode' they throw 'ExitCodeException' on failure.
     , runProcess_
-    , readProcessStdout
+    , readProcess_
     , readProcessStdout_
-    , readProcessStderr
     , readProcessStderr_
-    , readProcessInterleaved
     , readProcessInterleaved_
+    , withProcessWait_
+    , withProcessTerm_
+
+      -- | #interactwithaprocess#
 
       -- * Interact with a process
 
@@ -91,8 +117,16 @@ module System.Process.Typed
       -- * Exceptions
     , ExitCodeException (..)
     , ByteStringOutputException (..)
+
+      -- * Re-exports
+    , ExitCode (..)
+    , P.StdStream (..)
+
       -- * Unsafe functions
     , unsafeProcessHandle
+      -- * Deprecated functions
+    , withProcess
+    , withProcess_
     ) where
 
 import qualified Data.ByteString as S
@@ -108,7 +142,7 @@ import System.IO.Error (isPermissionError)
 import Control.Concurrent (threadDelay)
 import Control.Concurrent.Async (async, asyncWithUnmask, cancel, waitCatch)
 import Control.Concurrent.STM (newEmptyTMVarIO, atomically, putTMVar, TMVar, readTMVar, tryReadTMVar, STM, tryPutTMVar, throwSTM, catchSTM)
-import System.Exit (ExitCode (ExitSuccess))
+import System.Exit (ExitCode (ExitSuccess, ExitFailure))
 import System.Process.Typed.Internal
 import qualified Data.ByteString.Lazy as L
 import qualified Data.ByteString.Lazy.Char8 as L8
@@ -147,7 +181,11 @@ import qualified System.Process.Internals as P (createProcess_)
 --
 -- In all cases, the default for all three streams is to inherit the
 -- streams from the parent process. For other settings, see the
--- setters below for default values.
+-- [setters below](#processconfigsetters) for default values.
+--
+-- Once you have a @ProcessConfig@ you can launch a process from it
+-- using the functions in the section [Launch a
+-- process](#launchaprocess).
 --
 -- @since 0.1.0.0
 data ProcessConfig stdin stdout stderr = ProcessConfig
@@ -194,6 +232,7 @@ instance Show (ProcessConfig stdin stdout stderr) where
       where
         escape x
             | any (`elem` " \\\"'") x = show x
+            | x == "" = "\"\""
             | otherwise = x
 instance (stdin ~ (), stdout ~ (), stderr ~ ())
   => IsString (ProcessConfig stdin stdout stderr) where
@@ -210,7 +249,20 @@ instance (stdin ~ (), stdout ~ (), stderr ~ ())
 data StreamType = STInput | STOutput
 
 -- | A specification for how to create one of the three standard child
--- streams. See examples below.
+-- streams, @stdin@, @stdout@ and @stderr@. A 'StreamSpec' can be
+-- thought of as containing
+--
+-- 1. A type safe version of 'P.StdStream' from "System.Process".
+-- This determines whether the stream should be inherited from the
+-- parent process, piped to or from a 'Handle', etc.
+--
+-- 2. A means of accessing the stream as a value of type @a@
+--
+-- 3. A cleanup action which will be run on the stream once the
+-- process terminates
+--
+-- To create a @StreamSpec@ see the section [Stream
+-- specs](#streamspecs).
 --
 -- @since 0.1.0.0
 data StreamSpec (streamType :: StreamType) a = StreamSpec
@@ -243,6 +295,9 @@ instance Applicative Cleanup where
 -- | A running process. The three type parameters provide the type of
 -- the standard input, standard output, and standard error streams.
 --
+-- To interact with a @Process@ use the functions from the section
+-- [Interact with a process](#interactwithaprocess).
+--
 -- @since 0.1.0.0
 data Process stdin stdout stderr = Process
     { pConfig :: !(ProcessConfig () () ())
@@ -311,7 +366,9 @@ setShell cmd p = p { pcCmdSpec = P.ShellCommand cmd }
 --
 -- @since 0.1.0.0
 setStdin :: StreamSpec 'STInput stdin
+         -- ^
          -> ProcessConfig stdin0 stdout stderr
+         -- ^
          -> ProcessConfig stdin stdout stderr
 setStdin spec pc = pc { pcStdin = spec }
 
@@ -321,7 +378,9 @@ setStdin spec pc = pc { pcStdin = spec }
 --
 -- @since 0.1.0.0
 setStdout :: StreamSpec 'STOutput stdout
+          -- ^
           -> ProcessConfig stdin stdout0 stderr
+          -- ^
           -> ProcessConfig stdin stdout stderr
 setStdout spec pc = pc { pcStdout = spec }
 
@@ -331,7 +390,9 @@ setStdout spec pc = pc { pcStdout = spec }
 --
 -- @since 0.1.0.0
 setStderr :: StreamSpec 'STOutput stderr
+          -- ^
           -> ProcessConfig stdin stdout stderr0
+          -- ^
           -> ProcessConfig stdin stdout stderr
 setStderr spec pc = pc { pcStderr = spec }
 
@@ -341,7 +402,9 @@ setStderr spec pc = pc { pcStderr = spec }
 --
 -- @since 0.1.0.0
 setWorkingDir :: FilePath
+              -- ^
               -> ProcessConfig stdin stdout stderr
+              -- ^
               -> ProcessConfig stdin stdout stderr
 setWorkingDir dir pc = pc { pcWorkingDir = Just dir }
 
@@ -350,6 +413,7 @@ setWorkingDir dir pc = pc { pcWorkingDir = Just dir }
 -- @since 0.2.2.0
 setWorkingDirInherit
   :: ProcessConfig stdin stdout stderr
+  -- ^
   -> ProcessConfig stdin stdout stderr
 setWorkingDirInherit pc = pc { pcWorkingDir = Nothing }
 
@@ -359,7 +423,9 @@ setWorkingDirInherit pc = pc { pcWorkingDir = Nothing }
 --
 -- @since 0.1.0.0
 setEnv :: [(String, String)]
+       -- ^
        -> ProcessConfig stdin stdout stderr
+       -- ^
        -> ProcessConfig stdin stdout stderr
 setEnv env pc = pc { pcEnv = Just env }
 
@@ -368,6 +434,7 @@ setEnv env pc = pc { pcEnv = Just env }
 -- @since 0.2.2.0
 setEnvInherit
   :: ProcessConfig stdin stdout stderr
+  -- ^
   -> ProcessConfig stdin stdout stderr
 setEnvInherit pc = pc { pcEnv = Nothing }
 
@@ -379,7 +446,9 @@ setEnvInherit pc = pc { pcEnv = Nothing }
 -- @since 0.1.0.0
 setCloseFds
     :: Bool
+    -- ^
     -> ProcessConfig stdin stdout stderr
+    -- ^
     -> ProcessConfig stdin stdout stderr
 setCloseFds x pc = pc { pcCloseFds = x }
 
@@ -390,7 +459,9 @@ setCloseFds x pc = pc { pcCloseFds = x }
 -- @since 0.1.0.0
 setCreateGroup
     :: Bool
+    -- ^
     -> ProcessConfig stdin stdout stderr
+    -- ^
     -> ProcessConfig stdin stdout stderr
 setCreateGroup x pc = pc { pcCreateGroup = x }
 
@@ -402,7 +473,9 @@ setCreateGroup x pc = pc { pcCreateGroup = x }
 -- @since 0.1.0.0
 setDelegateCtlc
     :: Bool
+    -- ^
     -> ProcessConfig stdin stdout stderr
+    -- ^
     -> ProcessConfig stdin stdout stderr
 setDelegateCtlc x pc = pc { pcDelegateCtlc = x }
 
@@ -415,7 +488,9 @@ setDelegateCtlc x pc = pc { pcDelegateCtlc = x }
 -- @since 0.1.0.0
 setDetachConsole
     :: Bool
+    -- ^
     -> ProcessConfig stdin stdout stderr
+    -- ^
     -> ProcessConfig stdin stdout stderr
 setDetachConsole x pc = pc { pcDetachConsole = x }
 
@@ -426,7 +501,9 @@ setDetachConsole x pc = pc { pcDetachConsole = x }
 -- @since 0.1.0.0
 setCreateNewConsole
     :: Bool
+    -- ^
     -> ProcessConfig stdin stdout stderr
+    -- ^
     -> ProcessConfig stdin stdout stderr
 setCreateNewConsole x pc = pc { pcCreateNewConsole = x }
 
@@ -438,7 +515,9 @@ setCreateNewConsole x pc = pc { pcCreateNewConsole = x }
 -- @since 0.1.0.0
 setNewSession
     :: Bool
+    -- ^
     -> ProcessConfig stdin stdout stderr
+    -- ^
     -> ProcessConfig stdin stdout stderr
 setNewSession x pc = pc { pcNewSession = x }
 #endif
@@ -452,7 +531,9 @@ setNewSession x pc = pc { pcNewSession = x }
 -- @since 0.1.0.0
 setChildGroup
     :: GroupID
+    -- ^
     -> ProcessConfig stdin stdout stderr
+    -- ^
     -> ProcessConfig stdin stdout stderr
 setChildGroup x pc = pc { pcChildGroup = Just x }
 
@@ -461,6 +542,7 @@ setChildGroup x pc = pc { pcChildGroup = Just x }
 -- @since 0.2.2.0
 setChildGroupInherit
   :: ProcessConfig stdin stdout stderr
+  -- ^
   -> ProcessConfig stdin stdout stderr
 setChildGroupInherit pc = pc { pcChildGroup = Nothing }
 
@@ -472,7 +554,9 @@ setChildGroupInherit pc = pc { pcChildGroup = Nothing }
 -- @since 0.1.0.0
 setChildUser
     :: UserID
+    -- ^
     -> ProcessConfig stdin stdout stderr
+    -- ^
     -> ProcessConfig stdin stdout stderr
 setChildUser x pc = pc { pcChildUser = Just x }
 
@@ -481,6 +565,7 @@ setChildUser x pc = pc { pcChildUser = Just x }
 -- @since 0.2.2.0
 setChildUserInherit
   :: ProcessConfig stdin stdout stderr
+  -- ^
   -> ProcessConfig stdin stdout stderr
 setChildUserInherit pc = pc { pcChildUser = Nothing }
 #endif
@@ -489,23 +574,50 @@ setChildUserInherit pc = pc { pcChildUser = Nothing }
 -- helper function. This function:
 --
 -- * Takes as input the raw @Maybe Handle@ returned by the
--- 'P.createProcess' function. This will be determined by the
--- 'P.StdStream' argument.
+-- 'P.createProcess' function. The handle will be @Just@ 'Handle' if the
+-- 'P.StdStream' argument is 'P.CreatePipe' and @Nothing@ otherwise.
+-- See 'P.createProcess' for more details.
 --
 -- * Returns the actual stream value @a@, as well as a cleanup
--- * function to be run when calling 'stopProcess'.
+-- function to be run when calling 'stopProcess'.
+--
+-- If making a 'StreamSpec' with 'P.CreatePipe', prefer 'mkPipeStreamSpec',
+-- which encodes the invariant that a 'Handle' is created.
 --
 -- @since 0.1.0.0
 mkStreamSpec :: P.StdStream
+             -- ^
              -> (ProcessConfig () () () -> Maybe Handle -> IO (a, IO ()))
+             -- ^
              -> StreamSpec streamType a
 mkStreamSpec ss f = mkManagedStreamSpec ($ ss) f
 
+-- | Create a new 'P.CreatePipe' 'StreamSpec' from the given function.
+-- This function:
+--
+-- * Takes as input the @Handle@ returned by the 'P.createProcess' function.
+-- See 'P.createProcess' for more details.
+--
+-- * Returns the actual stream value @a@, as well as a cleanup
+-- function to be run when calling 'stopProcess'.
+--
+-- @since 0.2.10.0
+mkPipeStreamSpec :: (ProcessConfig () () () -> Handle -> IO (a, IO ()))
+                 -- ^
+                 -> StreamSpec streamType a
+                 -- ^
+mkPipeStreamSpec f = mkStreamSpec P.CreatePipe $ \pc mh ->
+    case mh of
+        Just h -> f pc h
+        Nothing -> error "Invariant violation: making StreamSpec with CreatePipe unexpectedly did not return a Handle"
+
 -- | Create a new 'StreamSpec' from a function that accepts a
 -- 'P.StdStream' and a helper function.  This function is the same as
 -- the helper in 'mkStreamSpec'
 mkManagedStreamSpec :: (forall b. (P.StdStream -> IO b) -> IO b)
+                    -- ^
                     -> (ProcessConfig () () () -> Maybe Handle -> IO (a, IO ()))
+                    -- ^
                     -> StreamSpec streamType a
 mkManagedStreamSpec ss f = StreamSpec ss (\pc mh -> Cleanup (f pc mh))
 
@@ -514,7 +626,7 @@ mkManagedStreamSpec ss f = StreamSpec ss (\pc mh -> Cleanup (f pc mh))
 --
 -- @since 0.1.0.0
 inherit :: StreamSpec anyStreamType ()
-inherit = mkStreamSpec P.Inherit (\_ Nothing -> pure ((), return ()))
+inherit = mkStreamSpec P.Inherit (\_ _ -> pure ((), return ()))
 
 -- | A stream spec which is empty when used for for input and discards
 -- output.  Note this requires your platform's null device to be
@@ -539,9 +651,9 @@ nullStream = mkManagedStreamSpec opener cleanup
 -- @since 0.1.0.0
 closed :: StreamSpec anyStreamType ()
 #if MIN_VERSION_process(1, 4, 0)
-closed = mkStreamSpec P.NoStream (\_ Nothing -> pure ((), return ()))
+closed = mkStreamSpec P.NoStream (\_ _ -> pure ((), return ()))
 #else
-closed = mkStreamSpec P.CreatePipe (\_ (Just h) -> ((), return ()) <$ hClose h)
+closed = mkPipeStreamSpec (\_ h -> ((), return ()) <$ hClose h)
 #endif
 
 -- | An input stream spec which sets the input to the given
@@ -550,7 +662,7 @@ closed = mkStreamSpec P.CreatePipe (\_ (Just h) -> ((), return ()) <$ hClose h)
 --
 -- @since 0.1.0.0
 byteStringInput :: L.ByteString -> StreamSpec 'STInput ()
-byteStringInput lbs = mkStreamSpec P.CreatePipe $ \_ (Just h) -> do
+byteStringInput lbs = mkPipeStreamSpec $ \_ h -> do
     void $ async $ do
         L.hPut h lbs
         hClose h
@@ -570,7 +682,7 @@ byteStringInput lbs = mkStreamSpec P.CreatePipe $ \_ (Just h) -> do
 --
 -- @since 0.1.0.0
 byteStringOutput :: StreamSpec 'STOutput (STM L.ByteString)
-byteStringOutput = mkStreamSpec P.CreatePipe $ \pc (Just h) -> byteStringFromHandle pc h
+byteStringOutput = mkPipeStreamSpec $ \pc h -> byteStringFromHandle pc h
 
 -- | Helper function (not exposed) for both 'byteStringOutput' and
 -- 'withProcessInterleave'. This will consume all of the output from
@@ -601,7 +713,7 @@ byteStringFromHandle pc h = do
 --
 -- @since 0.1.0.0
 createPipe :: StreamSpec anyStreamType Handle
-createPipe = mkStreamSpec P.CreatePipe $ \_ (Just h) -> return (h, hClose h)
+createPipe = mkPipeStreamSpec $ \_ h -> return (h, hClose h)
 
 -- | Use the provided 'Handle' for the child process, and when the
 -- process exits, do /not/ close it. This is useful if, for example,
@@ -610,7 +722,7 @@ createPipe = mkStreamSpec P.CreatePipe $ \_ (Just h) -> return (h, hClose h)
 --
 -- @since 0.1.0.0
 useHandleOpen :: Handle -> StreamSpec anyStreamType ()
-useHandleOpen h = mkStreamSpec (P.UseHandle h) $ \_ Nothing -> return ((), return ())
+useHandleOpen h = mkStreamSpec (P.UseHandle h) $ \_ _ -> return ((), return ())
 
 -- | Use the provided 'Handle' for the child process, and when the
 -- process exits, close it. If you have no reason to keep the 'Handle'
@@ -618,16 +730,17 @@ useHandleOpen h = mkStreamSpec (P.UseHandle h) $ \_ Nothing -> return ((), retur
 --
 -- @since 0.1.0.0
 useHandleClose :: Handle -> StreamSpec anyStreamType ()
-useHandleClose h = mkStreamSpec (P.UseHandle h) $ \_ Nothing -> return ((), hClose h)
+useHandleClose h = mkStreamSpec (P.UseHandle h) $ \_ _ -> return ((), hClose h)
 
 -- | Launch a process based on the given 'ProcessConfig'. You should
--- ensure that you close 'stopProcess' on the result. It's usually
+-- ensure that you call 'stopProcess' on the result. It's usually
 -- better to use one of the functions in this module which ensures
--- 'stopProcess' is called, such as 'withProcess'.
+-- 'stopProcess' is called, such as 'withProcessWait'.
 --
 -- @since 0.1.0.0
 startProcess :: MonadIO m
              => ProcessConfig stdin stdout stderr
+             -- ^
              -> m (Process stdin stdout stderr)
 startProcess pConfig'@ProcessConfig {..} = liftIO $ do
     ssStream pcStdin $ \realStdin ->
@@ -763,7 +876,9 @@ stopProcess = liftIO . pCleanup
 -- @since 0.2.5.0
 withProcessTerm :: (MonadUnliftIO m)
   => ProcessConfig stdin stdout stderr
+  -- ^
   -> (Process stdin stdout stderr -> m a)
+  -- ^
   -> m a
 withProcessTerm config = bracket (startProcess config) stopProcess
 
@@ -772,10 +887,15 @@ withProcessTerm config = bracket (startProcess config) stopProcess
 -- exit, and only kill it with 'stopProcess' in the event that the
 -- inner function throws an exception.
 --
+-- To interact with a @Process@ use the functions from the section
+-- [Interact with a process](#interactwithaprocess).
+--
 -- @since 0.2.5.0
 withProcessWait :: (MonadUnliftIO m)
   => ProcessConfig stdin stdout stderr
+  -- ^
   -> (Process stdin stdout stderr -> m a)
+  -- ^
   -> m a
 withProcessWait config f =
   bracket
@@ -795,10 +915,15 @@ withProcess = withProcessTerm
 
 -- | Same as 'withProcessTerm', but also calls 'checkExitCode'
 --
+-- To interact with a @Process@ use the functions from the section
+-- [Interact with a process](#interactwithaprocess).
+--
 -- @since 0.2.5.0
 withProcessTerm_ :: (MonadUnliftIO m)
   => ProcessConfig stdin stdout stderr
+  -- ^
   -> (Process stdin stdout stderr -> m a)
+  -- ^
   -> m a
 withProcessTerm_ config = bracket
     (startProcess config)
@@ -809,7 +934,9 @@ withProcessTerm_ config = bracket
 -- @since 0.2.5.0
 withProcessWait_ :: (MonadUnliftIO m)
   => ProcessConfig stdin stdout stderr
+  -- ^
   -> (Process stdin stdout stderr -> m a)
+  -- ^
   -> m a
 withProcessWait_ config f = bracket
     (startProcess config)
@@ -836,6 +963,7 @@ withProcess_ = withProcessTerm_
 -- @since 0.1.0.0
 readProcess :: MonadIO m
             => ProcessConfig stdin stdoutIgnored stderrIgnored
+            -- ^
             -> m (ExitCode, L.ByteString, L.ByteString)
 readProcess pc =
     liftIO $ withProcess pc' $ \p -> atomically $ (,,)
@@ -854,6 +982,7 @@ readProcess pc =
 -- @since 0.1.0.0
 readProcess_ :: MonadIO m
              => ProcessConfig stdin stdoutIgnored stderrIgnored
+             -- ^
              -> m (L.ByteString, L.ByteString)
 readProcess_ pc =
     liftIO $ withProcess pc' $ \p -> atomically $ do
@@ -874,6 +1003,7 @@ readProcess_ pc =
 readProcessStdout
   :: MonadIO m
   => ProcessConfig stdin stdoutIgnored stderr
+  -- ^
   -> m (ExitCode, L.ByteString)
 readProcessStdout pc =
     liftIO $ withProcess pc' $ \p -> atomically $ (,)
@@ -891,6 +1021,7 @@ readProcessStdout pc =
 readProcessStdout_
   :: MonadIO m
   => ProcessConfig stdin stdoutIgnored stderr
+  -- ^
   -> m L.ByteString
 readProcessStdout_ pc =
     liftIO $ withProcess pc' $ \p -> atomically $ do
@@ -909,6 +1040,7 @@ readProcessStdout_ pc =
 readProcessStderr
   :: MonadIO m
   => ProcessConfig stdin stdout stderrIgnored
+  -- ^
   -> m (ExitCode, L.ByteString)
 readProcessStderr pc =
     liftIO $ withProcess pc' $ \p -> atomically $ (,)
@@ -926,6 +1058,7 @@ readProcessStderr pc =
 readProcessStderr_
   :: MonadIO m
   => ProcessConfig stdin stdout stderrIgnored
+  -- ^
   -> m L.ByteString
 readProcessStderr_ pc =
     liftIO $ withProcess pc' $ \p -> atomically $ do
@@ -939,7 +1072,9 @@ readProcessStderr_ pc =
 
 withProcessInterleave :: (MonadUnliftIO m)
   => ProcessConfig stdin stdoutIgnored stderrIgnored
+  -- ^
   -> (Process stdin (STM L.ByteString) () -> m a)
+  -- ^
   -> m a
 withProcessInterleave pc inner =
     -- Create a pipe to be shared for both stdout and stderr
@@ -947,7 +1082,7 @@ withProcessInterleave pc inner =
         -- Use the writer end of the pipe for both stdout and stderr. For
         -- the stdout half, use byteStringFromHandle to read the data into
         -- a lazy ByteString in memory.
-        let pc' = setStdout (mkStreamSpec (P.UseHandle writeEnd) (\pc'' Nothing -> byteStringFromHandle pc'' readEnd))
+        let pc' = setStdout (mkStreamSpec (P.UseHandle writeEnd) (\pc'' _ -> byteStringFromHandle pc'' readEnd))
                 $ setStderr (useHandleOpen writeEnd)
                   pc
         withProcess pc' $ \p -> do
@@ -965,6 +1100,7 @@ withProcessInterleave pc inner =
 readProcessInterleaved
   :: MonadIO m
   => ProcessConfig stdin stdoutIgnored stderrIgnored
+  -- ^
   -> m (ExitCode, L.ByteString)
 readProcessInterleaved pc =
     liftIO $
@@ -982,7 +1118,9 @@ readProcessInterleaved pc =
 readProcessInterleaved_
   :: MonadIO m
   => ProcessConfig stdin stdoutIgnored stderrIgnored
+  -- ^
   -> m L.ByteString
+  -- ^
 readProcessInterleaved_ pc =
     liftIO $
     withProcessInterleave pc $ \p -> atomically $ do
@@ -998,7 +1136,9 @@ readProcessInterleaved_ pc =
 -- @since 0.1.0.0
 runProcess :: MonadIO m
            => ProcessConfig stdin stdout stderr
+           -- ^
            -> m ExitCode
+           -- ^
 runProcess pc = liftIO $ withProcess pc waitExitCode
 
 -- | Same as 'runProcess', but instead of returning the
@@ -1007,6 +1147,7 @@ runProcess pc = liftIO $ withProcess pc waitExitCode
 -- @since 0.1.0.0
 runProcess_ :: MonadIO m
             => ProcessConfig stdin stdout stderr
+            -- ^
             -> m ()
 runProcess_ pc = liftIO $ withProcess pc checkExitCode
 
